/*
    Fichier filtrechamp.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme dtruit les fichiers mail comportant une chaine de
    caractres particulire dans le champ.

    La liste des chaines de caractres interdites est mmorise dans
    un fichier

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "pop.h"
#include "testchamp.h"
#include "trtentete.h"
#include "szchemin.h"


#define nbmax_champ   500     // nombre max de champs interdits


/* prototypes */
int testchamp (int numes, char *nomchamp);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

char *listechamp [nbmax_champ]; // mmorise les champs interdits
int  sz_listechamp;     // nombre de champs mmoriss dans listechamp
char bufChamp [80];     // buffer contenant le champ du mail  tester
FILE *ftrace;           // Fichier contenant les champs des mail refuss
char datecour [10];     // Date courante (pour le fichier des mails refuss)


/* programme principal */

int main (int nbarg, char *varg[])
{
    char ficonf [80];    // fichier de configuration du systme de mail
    FILE *fconf;         // descripteur du fichier de configuration
    char fichliste [szchemin]; // fichier contenant la liste des chaines  tester
    char *nomchamp;      // nom du champ  tester
    char *nomfliste;     // nom du fichier contenant les chaines  tester
    int  numes, nbmes;   // mumro du mail courant et nombre de mails
    char bufw [80];      // buffer d'envoi d'une requte de destruction de mail
    int  conserves = 0;  // nombre de mails conservs
    int  supprimes = 0;  // nombre de mails supprims
    char *ncom;          // nom de la commande que l'on excute (filtrechamp)
    int  i, j;           // compteurs


    // rcupration du nom de l'excutable
    ncom = varg [0];
    nbarg --;
    varg ++;

    // rcupration du nom du champ  tester
    if (nbarg > 1)
    {
        nomchamp = varg [0];
        nbarg --;
        varg ++;
    }

    // initialisations valeur des options
    ftrace = NULL;
    nomfliste = NULL;

    // si le programme a t lanc avec des options
    while (nbarg >= 3 && **varg == '-')
    {
        // traitement des 2 options
        switch (varg [0][1])
        {
            // option trace des mails supprims
            case 't' : // ouvrir le fichier trace en criture fin de fichier
                       ftrace = fopen (varg [1], "a");

                       // si nom de fichier correct
                       if (ftrace)
                           // initialiser la date courante
                           initdatecour (datecour);
                       else
                           // sinon, avertir l'utilisateur
                           fprintf (stderr,
                    "Impossible d'crire dans le fichier %s\n", varg [1]);

                       break;

            // option nom du fichier contenant la liste des chaines autorises
            case 'f' : nomfliste = varg [1];
                       break;

            // aucune autre option n'est reconnue
            default  : fprintf (stderr, "Option %s incorrecte\n", *varg);
                       nbarg = 0;  // pour rappeller la syntaxe
        }

        // sauter les arguments traits
        nbarg -= 2;
        varg += 2;
    }

    // controle du nombre d'arguments restants
    if (nbarg == 1)
    {
        // rcuprer le nom du fichier de configuration
        strcpy (ficonf, *varg);

        if (ficonf [strlen (ficonf) - 4] != '.')
            strcat (ficonf, ".cfg");

        // ouvrir le fichier de configuration
        fconf = fopen (ficonf, "r");

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // dtermination du nom du fichier
                // contenant les chaines interdites

                // cas particulier : chemin d'accs absolu  ce fichier
                if (nomfliste && *nomfliste == '/')
                    strcpy (fichliste, nomfliste);
                else
                {
                    // cas gnral : pas de chemin d'accs absolu au fichier

                    // rcupration du rpertoire racine de la messagerie
                    fgets (fichliste, sizeof (fichliste), fconf);

                    // on complte ce nom d'un /
                    strcpy (fichliste + strlen (fichliste) - 1, "/");

                    // si le nom de ce fichier a t pass en paramtre
                    if (nomfliste)
                        // le rajouter au nom du rpertoire
                        strcat (fichliste, nomfliste);
                    else
                    {
                        // sinon, on va gnrer un nom  partir du nom du champ
                        strcat (fichliste, "refus-");

                        i = strlen (fichliste);
                        j = 0;

                        do
                            // le nom du champ est recopi en minuscules
                            fichliste [i++] = tolower (nomchamp [j]);
                        while (nomchamp [j++]);
                    }
                }

                // charger en mmoire les champs interdits
                sz_listechamp = charge_valchamp (fichliste, listechamp,
                                                 nbmax_champ, "nbmax_champ");

                // si la liste n'est pas vide
                if (sz_listechamp)
                {
                    // rcupration du nombre de mails
                    nbmes = nbmails ();

                    // vrification des diffrents mails
                    for (numes = 1; numes <= nbmes; numes++)
                    {
                        // si le champ du mail contient une chaine interdite
                        if (testchamp (numes, nomchamp))
                        {
                            // demande de destruction du mail
                            sprintf (bufw, "DELE %d", numes);
                            env_pop (bufw);
                            lire_pop ();
                            supprimes ++;

                            // si utilisation d'un fichier trace
                            if (ftrace)
                                // mmoriser le champ de ce mail
                                fprintf (ftrace, "%s%s\n", datecour, bufChamp);
                        }
                        else
                            conserves ++;
                    }

                    // dition d'un rcapitulatif
                    printf ("\n%d messages conservs, %d supprims\n",
                                              conserves, supprimes);
                }

                // se dconnecter proprement du serveur pop
                deconnect_pop ();

                // si des mails ont t supprims
                if (supprimes)
                    // attendre pour enregistrement correct des suppressions
                    // (vite problme si autre filtre appel juste aprs)
                    sleep (2);
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
        else
            fprintf (stderr, "Fichier %s non trouv\n", ficonf);
    }
    else
        fprintf (stderr,
 "Syntaxe : %s nom_champ [-f fich_liste] [-t fich_trace] fich_configuration\n",
                                                                          ncom);

    return (0);
}



/* lit l'entte d'un mail et vrifie si son champ est autoris */

int testchamp (int numes, char *nomchamp)
{
    char bufw [80];       // buffer d'envoi d'une requte
    int  i;               // simple compteur


    printf ("\rMessage %d", numes);
    fflush (stdout);

    // initialisation
    bufChamp [0] = '\0';

    // demande de lecture de l'entte du message
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lecture entte du message et mmorisation de le champ du mail
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // si on a trouv le champ choisi
        if (start (nomchamp))
        {
            // convertir les caractres spciaux de la ligne
            majlignentete ();

            // initialisation
            i = strlen (nomchamp) + 1;

            // recherche du dbut du champ
            while (buf_lect [i] == ' ')
                i++;

            // tronquer si ncessaire le champ (pour viter un plantage)
            if (strlen (buf_lect + i) > sizeof (bufChamp))
                 buf_lect [i + sizeof (bufChamp) - 1] = '\0';

            // mmorisation du champ
            strcpy (bufChamp, buf_lect + i);
        }
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // vrifier si le mail contient un des champ de la liste interdite
    return trouve_valchamp (bufChamp, listechamp, sz_listechamp);
}
